#pragma once

#include <memory>
#include <filesystem>

#include "AudioManager.h"
#include "FileLogger.h"
#include "DatabaseManager.h"
#include "NetworkManager.h"
#include "CommandLineArgs.h"

class MainApplication
{
private:

    CommandLineArgsUPtr mCommandLineArgs;
    DatabaseManagerPtr mDatabaseManager;
    NetworkManagerUPtr mNetworkManager;
    AudioManagerUPtr mAudioManager;

public:

    MainApplication();

    ~MainApplication();

    void Initialize(CommandLineArgsUPtr commandLineArgs);

    void Run();

    void ShutDown();

    CommandLineArgs* GetCommandLineArgs() const;

    static std::shared_ptr<MainApplication> Create();

private:
    void RunServer();

    void PlayAudio();

    void ConvertDocument(const std::string& inputPath, const std::string& outputPath);
};

using MainApplicationPtr = std::shared_ptr<MainApplication>;
