#include "Window.h"

namespace mt{
Window::Window()
    :mWidth(400),
     mHeight(300),
     mWidget()
{

}

Window::~Window()
{

}

std::unique_ptr<Window> Window::Create()
{
    return std::make_unique<Window>();
}

std::vector<VisualLayer*> Window::GetLayers()
{
    return mLayers;
}

void Window::OnMouseEvent(const MouseEvent* event)
{
    mWidget->OnMouseEvent(event);
}

void Window::OnKeyboardEvent(const KeyboardEvent* event)
{
    mWidget->OnKeyboardEvent(event);
}

void Window::OnPaint(const PaintEvent* event)
{
    mLayers.clear();
    mWidget->SetBounds(mWidth, mHeight);
    mWidget->OnPaintEvent(event);

    auto layers = mWidget->GetLayers();
    mLayers.insert(mLayers.end(), layers.begin(), layers.end());
}

void Window::AddWidget(WidgetUPtr widget)
{
    mWidget = std::move(widget);
}

Widget* Window::GetWidget() const
{
    return mWidget.get();
}

unsigned Window::GetWidth() const
{
    return mWidth;
}

unsigned Window::GetHeight() const
{
    return mHeight;
}

void Window::SetSize(unsigned width, unsigned height)
{
    mWidth = width;
    mHeight = height;
}
}
