#pragma once

#include <memory>
#include <vector>

#include "PaintEvent.h"
#include "MouseEvent.h"
#include "KeyboardEvent.h"
#include "VisualLayer.h"
#include "Widget.h"

namespace mt
{

class Window
{

private:

    WidgetUPtr mWidget;
    unsigned mWidth;
    unsigned mHeight;
    std::vector<VisualLayer*> mLayers;

public:

    Window();

    ~Window();

    static std::unique_ptr<Window> Create();

    void AddWidget(WidgetUPtr widget);

    Widget* GetWidget() const;

    std::vector<VisualLayer*> GetLayers();

    unsigned GetWidth() const;

    unsigned GetHeight() const;

    void SetSize(unsigned width, unsigned height);

    void OnPaint(const PaintEvent* event);

    void OnMouseEvent(const MouseEvent* event);

    void OnKeyboardEvent(const KeyboardEvent* event);
};
}

using WindowUPtr = std::unique_ptr<mt::Window>;
