#pragma once

#include <vector>
#include <memory>

#include "Keyboard.h"
#include "Screen.h"
#include "KeyboardEvent.h"
#include "PaintEvent.h"
#include "MouseEvent.h"
#include "WindowManager.h"
#include "UiEvent.h"
#include "AbstractDesktopApp.h"
#include "EventManager.h"

class DesktopManager
{
    std::vector<ScreenPtr> mScreens;
    WindowManagerUPtr mWindowManager;
    KeyboardUPtr mKeyboard;
    EventManagerUPtr mEventManager;
    AbstractDesktopAppPtr mMainApplication;
    bool mModified;

public:

    DesktopManager();

    ~DesktopManager();

    static std::unique_ptr<DesktopManager> Create();

    void SetWindowManager(WindowManagerUPtr windowManager);

    void SetMainApp(AbstractDesktopAppPtr mainApp);

    AbstractDesktopAppPtr GetMainApp();

    WindowManager* GetWindowManager() const;

    Keyboard* GetKeyboard() const;

    bool IsModified() const;

    void SetIsModified(bool modified);

    void SetKeyboard(KeyboardUPtr keyboard);

    void OnUiEvent(UiEventUPtr event);

    void OnKeyboardEvent(const KeyboardEvent* keyboardEvent);

    void OnMouseEvent(const MouseEvent* mouseEvent);

    void OnPaintEvent(const PaintEvent* paintEvent);

    void ClearEvents();
};

using DesktopManagerUPtr = std::unique_ptr<DesktopManager>;
