#pragma once

#include <memory>
#include <map>

#include "Window.h"
#include "VisualLayer.h"

class DesktopManager;
using DesktopManagerPtr = std::shared_ptr<DesktopManager>;

class GlxInterface;
using GlxInterfacePtr = std::shared_ptr<GlxInterface>;

struct xcb_screen_t;
struct xcb_connection_t;
struct xcb_key_press_event_t;
struct xcb_button_press_event_t;
struct _XDisplay;

class XcbInterface
{

private:

    bool mUseOpenGl;
    std::map<unsigned, mt::Window*> mWindows;
    std::map<mt::Window*, unsigned> mHandles;
    unsigned mDefaultWindow;
    xcb_connection_t* mConnection;
    xcb_screen_t* mScreen;
    unsigned mGraphicsContext;
    _XDisplay* mX11Display;
    GlxInterfacePtr mGlxInterface;

public:

    XcbInterface();

    ~XcbInterface();

    void SetUseOpenGl(bool use);
    void onPaint(DesktopManager* desktopManager);

    void Initialize();

    void InitializeOpenGl();

    void CreateOpenGlDrawable(mt::Window* window);

    void Loop(DesktopManager* desktopManager);

    void LoopOpenGl(DesktopManager* desktopManager);

    void ShutDown();

    void AddWindow(mt::Window* window);

    void ShowWindow(mt::Window* window);

    void PaintWindow(mt::Window* window);

    void ClearWindow(mt::Window* window);

private:

    void Connect();

    void UpdateScreen();

    uint32_t GetEventMask();

    void OnKeyPress(xcb_key_press_event_t* event, DesktopManager*);

    void OnKeyRelease(xcb_key_press_event_t* event, DesktopManager*);

    void OnButtonPress(xcb_button_press_event_t* event, DesktopManager*);

    void OnButtonRelease(xcb_button_press_event_t* event, DesktopManager*);

    void MapWindow(mt::Window* window);

    void CreateGraphicsContext();


};
